/*
 *
 * Copyright  2004, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */

package cse466.packets;


public class SangSongPacket extends CSE466Packet {
  public int transmittingNodeNum;
  public int sequenceNum;
  public int songNum;
  public int songWeight;
  public int weightMaxSongNum;
  public int weightMax;
  public int topSong2Num;
  public int topSong2Weight;
  public int weightMinSongNum;
  public int weightMin;
  public int topNodeNum;
  public int topNodeStrength;


  public SangSongPacket() {
    transmittingNodeNum = 4;
    sequenceNum = 5;
    songNum = 6;
    songWeight = 7;
    weightMaxSongNum = 8;
    weightMax = 9;
    topSong2Num = 10;
    topSong2Weight = 11;
    weightMinSongNum = 12;
    weightMin = 13;
    topNodeNum = 14;
    topNodeStrength = 15;
  }

  public int getProperAMType(){
    return 42;
  }


  public byte[] setFromBytes(byte[] fromArray){
    fromArray = super.setFromBytes(fromArray);
    transmittingNodeNum   = bytesToInt(fromArray[ 0], fromArray[ 1]);
    sequenceNum           = bytesToInt(fromArray[ 2], fromArray[ 3]);
    songNum               = bytesToInt(fromArray[ 4], fromArray[ 5]);
    songWeight            = bytesToInt(fromArray[ 6], fromArray[ 7]);
    weightMaxSongNum      = bytesToInt(fromArray[ 8], fromArray[ 9]);
    weightMax             = bytesToInt(fromArray[10], fromArray[11]);
    topSong2Num           = bytesToInt(fromArray[12], fromArray[13]);
    topSong2Weight        = bytesToInt(fromArray[14], fromArray[15]);
    weightMinSongNum      = bytesToInt(fromArray[16], fromArray[17]);
    weightMin             = bytesToInt(fromArray[18], fromArray[19]);
    topNodeNum            = bytesToInt(fromArray[20], fromArray[21]);
    topNodeStrength       = bytesToInt(fromArray[22], fromArray[23]);
    return subArray(fromArray, 24);
  }

  protected byte[] getByteArray(byte[] childByteArray){
    byte[] ret = new byte[childByteArray.length + 24];
    ret[ 0] = intToLSB(transmittingNodeNum);
    ret[ 1] = intToMSB(transmittingNodeNum);
    ret[ 2] = intToLSB(sequenceNum);
    ret[ 3] = intToMSB(sequenceNum);
    ret[ 4] = intToLSB(songNum);
    ret[ 5] = intToMSB(songNum);
    ret[ 6] = intToLSB(songWeight);
    ret[ 7] = intToMSB(songWeight);
    ret[ 8] = intToLSB(weightMaxSongNum);
    ret[ 9] = intToMSB(weightMaxSongNum);
    ret[10] = intToLSB(weightMax);
    ret[11] = intToMSB(weightMax);
    ret[12] = intToLSB(topSong2Num);
    ret[13] = intToMSB(topSong2Num);
    ret[14] = intToLSB(topSong2Weight);
    ret[15] = intToMSB(topSong2Weight);
    ret[16] = intToLSB(weightMinSongNum);
    ret[17] = intToMSB(weightMinSongNum);
    ret[18] = intToLSB(weightMin);
    ret[19] = intToMSB(weightMin);
    ret[20] = intToLSB(topNodeNum);
    ret[21] = intToMSB(topNodeNum);
    ret[22] = intToLSB(topNodeStrength);
    ret[23] = intToMSB(topNodeStrength);
    copyIntoArray(ret, childByteArray, 24);
    return super.getByteArray(ret);
  }

  public byte[] getByteArray(){
    return getByteArray(new byte[0]);
  }

  public void print(){
    super.print();
    System.out.println("SS     transmittingNodeNum : " + Integer.toHexString(transmittingNodeNum));
    System.out.println("SS     sequenceNum         : " + Integer.toHexString(sequenceNum));
    System.out.println("SS     songNum             : " + Integer.toHexString(songNum));
    System.out.println("SS     songWeight          : " + Integer.toHexString(songWeight));
    System.out.println("SS     weightMaxSongNum    : " + Integer.toHexString(weightMaxSongNum));
    System.out.println("SS     weightMax           : " + Integer.toHexString(weightMax));
    System.out.println("SS     topSong2Num         : " + Integer.toHexString(topSong2Num));
    System.out.println("SS     topSong2Weight      : " + Integer.toHexString(topSong2Weight));
    System.out.println("SS     weightMinSongNum    : " + Integer.toHexString(weightMinSongNum));
    System.out.println("SS     weightMin           : " + Integer.toHexString(weightMin));
    System.out.println("SS     topNodeNum          : " + Integer.toHexString(topNodeNum));
    System.out.println("SS     topNodeStrength     : " + Integer.toHexString(topNodeStrength));
  }

  public boolean equals(SangSongPacket other){
    if(transmittingNodeNum != other.transmittingNodeNum) return false;
    if(sequenceNum         != other.sequenceNum        ) return false;
    if(songNum             != other.songNum            ) return false;
    if(songWeight          != other.songWeight         ) return false;
    if(weightMaxSongNum    != other.weightMaxSongNum   ) return false;
    if(weightMax           != other.weightMax          ) return false;
    if(topSong2Num         != other.topSong2Num        ) return false;
    if(topSong2Weight      != other.topSong2Weight     ) return false;
    if(weightMinSongNum    != other.weightMinSongNum   ) return false;
    if(weightMin           != other.weightMin          ) return false;
    if(topNodeNum          != other.topNodeNum         ) return false;
    if(topNodeStrength     != other.topNodeStrength    ) return false;
    return super.equals(other);
  }
}

